package com.example.ydsdkdemo

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.SystemClock
import android.util.Log
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.Toast
import com.example.ydsdkdemo.util.SharedPreferencesUtils
import com.example.ydsdkdemo.util.Util
import com.open.ad.polyunion.bean.AdSource
import com.open.ad.polyunion.bean.CAdInfo
import com.open.ad.polyunion.listener.AdViewListener
import com.open.ad.polyunion.view.AdConfig
import com.open.ad.polyunion.view.AdRequestConfig
import com.open.ad.polyunion.view.AdView
import com.example.ydsdkdemo.BuildConfig

import com.example.ydsdkdemo.R
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class SplashActivity : Activity() {

    // 应用设定开屏界面展示最短时间(毫秒)
    val MIN_SPLASH_SHOWING_MILLIS: Long = 3500

    val TAG = "cloooudAD"
    private var splash: AdView? = null
    private var mContainer: RelativeLayout? = null
    private var mAppLogo: ImageView? = null

    // 开屏界面开始展示时间
    private var mSplashStartTimeStamp: Long = 0

    // 是否立马跳转到主界面
    private var canJumpImmediately = false

    // activity是否可见
    private var isActivityVisible = false

    private var isShowDownloadConfirmDialog = false
    private var isWholeScreenClickable = false
    private var IS_FINISH = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_splash)

        IS_FINISH = intent.getBooleanExtra("IS_FINISH", false)

        isShowDownloadConfirmDialog =
            SharedPreferencesUtils.getParam(this, "isShowDownloadConfirmDialog", true) as Boolean
        isWholeScreenClickable =
            SharedPreferencesUtils.getParam(this, "isWholeScreenClickable", true) as Boolean

        mContainer = findViewById(R.id.adContainer)
        mAppLogo = findViewById(R.id.appLogo)
        showNormalSplash()
    }

    private fun showNormalSplash() {
        val widthPixels = resources.displayMetrics.widthPixels
        val heightPixels = resources.displayMetrics.heightPixels
        val screenSizeDP: IntArray = Util.getScreenSizeInDP(applicationContext)
        val aspectRatio = heightPixels / widthPixels.toFloat()
        Log.e(TAG, "showNormalSplash: "+System.currentTimeMillis() )
        mSplashStartTimeStamp = SystemClock.elapsedRealtime()
        val requestConfig: AdRequestConfig = AdRequestConfig.Builder()
            .AdType(AdConfig.AD_TYPE_SPLASH)
            .slotId(BuildConfig.SPLASH_SLOT_ID)//广告位id
//            .sceneId("2323524") //广告场景id
            .isWholeScreenClickable(isWholeScreenClickable) // 头条的开屏广告控制哪儿点击
            .requestTimeOutMillis(50000) //设置聚合SDK请求前超时时间，毫秒
//            .gdtSplashTimeoutMillis(3600) //设置广点通SDK请求超时时间，毫秒
//            .toutiaoSplashTimeoutMillis(3600) //设置穿山甲SDK请求超时时间，毫秒
            .widthDp(screenSizeDP[0]) //请求图片宽度dp值
            .heightDp(screenSizeDP[1]) //需要传广告展示的高度
            .splashContainer(mContainer) //设置开屏广告view容器
            .showDownloadConfirmDialog(isShowDownloadConfirmDialog)
            .build()
        splash = AdView(this, requestConfig)
        splash!!.setListener(object : AdViewListener {
            override fun onAdReady(price: Int) {
                Log.e(TAG, "onAdReady: "+System.currentTimeMillis() )
                Log.e("CLOOOUDAD", "showNormalSplash:onAdReady")
                if (splash!!.adSource === AdSource.CLOOOUD &&
                    splash!!.splashAdMaterialType !== AdConfig.MATERIAL_TYPE_IMAGE
                ) {
                    mAppLogo!!.visibility = View.GONE
                }
                Log.e("CLOOOUD_POLY", "onAdReady: ")

//                //加载开屏广告
                splash!!.showSplashAd()
            }

            override fun onAdShow(cAdInfo: CAdInfo?) {
                Log.e("CLOOOUD_POLY", "ecpm: " + cAdInfo?.ecpm)
                Log.e(TAG, "onAdShow: "+System.currentTimeMillis() )
                Log.e("CLOOOUDAD", "showNormalSplash:onAdShow")
            }

            override fun onAdClick() {
                Log.e("CLOOOUDAD", "showNormalSplash:onAdClick: ")
            }

            override fun onAdFailed(s: String) {
                Log.e("CLOOOUDAD", ":onAdFailed: "+s)
                Toast.makeText(this@SplashActivity, "广告请求失败: $s", Toast.LENGTH_SHORT).show()
                delayJumpToMainActivity()
            }

            override fun onAdDismissed(s: String?) {
                if (isActivityVisible) {
                    goToMainActivity()
                }
            }
        })
        if (IS_FINISH) {
            Handler(Looper.getMainLooper()).postDelayed({ finish() }, 200)
        }
    }

    /**
     * 延迟跳转到主页，避免广告加载失败，开屏页立马闪过
     */
    private fun delayJumpToMainActivity() {
        var leftTimeInMillis: Long =
             MIN_SPLASH_SHOWING_MILLIS - (SystemClock.elapsedRealtime() - mSplashStartTimeStamp)
        leftTimeInMillis = if (leftTimeInMillis >= 100) leftTimeInMillis else 100
        Handler(Looper.getMainLooper()).postDelayed({ goToMainActivity() }, leftTimeInMillis)
    }

    override fun onPause() {
        super.onPause()
        isActivityVisible = false
        canJumpImmediately = true
    }

    override fun onResume() {
        super.onResume()
        isActivityVisible = true
        if (canJumpImmediately) {
            goToMainActivity()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        splash?.onDestroyAd()
    }

    private fun goToMainActivity() {
        this.startActivity(Intent(this@SplashActivity, MainActivity::class.java))
        finish()
    }
}