package com.example.ydsdkdemo.ui

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import android.widget.Toast
import com.example.ydsdkdemo.util.GlideTool
import com.example.ydsdkdemo.util.Util
import com.open.ad.polyunion.bean.CAdInfo
import com.open.ad.polyunion.listener.CAppDownloadListener
import com.open.ad.polyunion.listener.NativeAdsListener
import com.open.ad.polyunion.view.AdConfig
import com.open.ad.polyunion.view.AdRequestConfig
import com.open.ad.polyunion.view.NativeAd
import com.open.ad.polyunion.view.NativeAdsResponse
import com.open.ad.polyunion.view.NativeSelfRenderAdContainer
import com.example.ydsdkdemo.R

/**
 * @date :2024/5/13 15:14
 * @projectName
 * @author zw
 * @description:
 */
class ShowAdDialog(context: Context) : Dialog(context) {

    private val TAG = "CLOOOUD_TAG"
    private var mContext: Context? = null
    private var adContainer: NativeSelfRenderAdContainer? = null
    private var adExpressContainer: RelativeLayout? = null

    protected var adPackageName: TextView? = null
    protected var title: TextView? = null
    protected var desc: TextView? = null
    protected var adIcon: ImageView? = null
    protected var downloadState: TextView? = null
    protected var adCta: TextView? = null

    init {
        mContext = context
        showAd()
        window!!.setBackgroundDrawable(ColorDrawable(0x00000000))
        window!!.setLayout(
            WindowManager.LayoutParams.MATCH_PARENT,
            WindowManager.LayoutParams.WRAP_CONTENT
        )
    }


    fun showAd() {
        val screenSizeDP: IntArray = Util.getScreenSizeInDP(mContext)
        val config: AdRequestConfig = AdRequestConfig.Builder()
            .sceneId("")
            .widthDp(screenSizeDP[0]) //请求图片宽度dp值
            .heightDp(0) //请求图片高度dp值
            .requestCount(1) //信息流广告请求条数
            .isVideoVoiceOn(false)
            /**
             * 设置视频自动播放场景：
             * ALWAYS - 所有网络环境均自动播放
             * WIFI - WIFI网络环境自动播放
             * NEVER - 所有网络环境均不会自动播放
             */
            .videoAutoPlayPolicy(AdRequestConfig.VideoAutoPlayPolicy.ALWAYS) //                .addSougouAdTemplate(AdTemplate.BIG_IMG_TPL_ID)
            //                .addSougouAdTemplate(AdTemplate.DOWNLOAD_BIG_TPL_ID)
            .showDownloadConfirmDialog(true)
            .isAdNeedRemoveDuplicates(true)
            .build()
        val nativeAds = NativeAd(context.applicationContext, config)
        nativeAds.setListener(object : NativeAdsListener {
            override fun onAdReady(arrayList: List<NativeAdsResponse>?) {
                if (arrayList.isNullOrEmpty()) {
                    return
                }
                val nativeAdResponse: NativeAdsResponse = arrayList[0]
                initView(nativeAdResponse)
            }

            override fun onAdFailed(s: String) {
                Log.e("CLOOOUDAD", "NativeAd onAdFailed: $s")
            }
        })
    }

    private fun initView(response: NativeAdsResponse) {
        response.setNativeActionListener(object : NativeAdsResponse.NativeActionListener {
            override fun onAdClick() {
                Log.e("CLOOOUDAD", "NativeAd onAdClick")
            }

            override fun onAdShow(cAdInfo: CAdInfo) {
                Log.e("CLOOOUDAD", "NativeAd onAdShow")
            }

            override fun onError(s: String) {
                Log.e("CLOOOUDAD", "NativeAd onError: $s")
            }

            override fun onAdClose(s: String) {
                Log.i(TAG, "onAdLoaded=====回调方法运行了")
                Log.e("CLOOOUDpoly", "NativeAd onAdClose: $s")
                onAdClosed(response)
            }

            override fun onExpressRenderFail(s: String) {
                //渲染失败
                Toast.makeText(mContext, "广告渲染失败: $s", Toast.LENGTH_SHORT).show()
            }

            override fun onExpressRenderSuccess(view: View?, v: Float, v1: Float) {
                // 渲染成功
            }

            override fun onExpressRenderTimeout() {
                // 渲染超时
            }

            override fun onDislikeDialogShow() {}
            override fun onDownloadConfirmDialogDismissed() {}
        })
        Log.i(TAG, "判断广告类型=======" + response.isExpressAd)
        if (response.isExpressAd) {
            // 模板广告开始渲染
            response.render()
        } else if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD) {
            // 自渲染下载类广告，可设置下载回调，刷新自渲染下载按钮 (针对应用自渲染下载按钮的场景)
            response.appDownloadListener = object : CAppDownloadListener {
                override fun onIdle() {
                    updateDownloadState("立即下载")
                }

                override fun onProgressUpdate(i: Int) {
                    updateDownloadState("下载 $i%")
                }

                override fun onDownloadPaused() {
                    updateDownloadState("继续下载")
                }

                override fun onDownloadFinished() {
                    updateDownloadState("点击安装")
                }

                override fun onDownloadFailed() {
                    updateDownloadState("下载失败")
                }

                override fun onInstalled() {
                    updateDownloadState("点击打开")
                }
            }
        }
        if (response.isExpressAd) {
            // LIST_DATA_AD_EXPRESS
            bindAd(response, 0)
        } else {
            Log.i(TAG, "getSelfRenderAdMediaType=======" + response.selfRenderAdMediaType)
            Log.i(TAG, "getSelfRenderAdStyle=======" + response.selfRenderAdStyle)
            when (response.selfRenderAdMediaType) {
                NativeAdsResponse.NATIVE_AD_MEDIA_TYPE_IMAGE -> when (response.selfRenderAdStyle) {
                    NativeAdsResponse.NATIVE_AD_STYLE_LARGE_IMAGE -> {
                        // 指定大图样式
                        //LIST_DATA_SELF_RENDER_AD_LARGE_IMAGE
                        bindAd(response, 1)
                        return
                    }

                    NativeAdsResponse.NATIVE_AD_STYLE_SMALL_IMAGE -> {
                        // 指定小图样式(左图右文或者右图作文)
                        //LIST_DATA_SELF_RENDER_AD_SMALL_IMAGE
                        bindAd(response, 2)
                        return
                    }

                    NativeAdsResponse.NATIVE_AD_STYLE_GROUP_IMAGE -> {
                        // 指定三图样式
                        //LIST_DATA_SELF_RENDER_AD_GROUP_IMAGE
                        bindAd(response, 3)
                        return
                    }

                    else -> {
                        val imageNum: Int = response.imageUrls.size
                        if (imageNum < 3) {
                            //LIST_DATA_SELF_RENDER_AD_LARGE_IMAGE
                            bindAd(response, 1)
                            return
                        } else {
                            //LIST_DATA_SELF_RENDER_AD_GROUP_IMAGE
                            bindAd(response, 3)
                            return
                        }
                    }
                }

                NativeAdsResponse.NATIVE_AD_MEDIA_TYPE_VIDEO ->                     // LIST_DATA_SELF_RENDER_AD_VIDEO
                    bindAd(response, 4)
            }
            // LIST_DATA_SELF_RENDER_AD_LARGE_IMAGE
            bindAd(response, 1)
        }
    }

    private fun bindAd(response: NativeAdsResponse, type: Int) {
        Log.i(TAG, "bindAd========$type")
        when (type) {
            0 -> {
                setContentView(R.layout.listitem_ad_native_express)
                adExpressContainer = findViewById<RelativeLayout>(R.id.adExpressContainer)
                val container = findViewById<View>(R.id.iv_listitem_express) as FrameLayout?
                val mAdSource: TextView = findViewById<TextView>(R.id.mAdSource)
                response.setTTDefaultDislikeDialog(mContext as Activity?) //设置穿山甲SDK的dislike弹窗
                mAdSource.text = response.adSource.getName()
                if (container != null) {
                    //获取模板view
                    val video: View = response.getExpressAdView(context)
                    if (video != null) {
                        if (video.parent != null) {
                            val viewGroup = video.parent as ViewGroup
                            viewGroup.removeView(video)
                        }
                        container.removeAllViews()
                        val lp = FrameLayout.LayoutParams(
                            FrameLayout.LayoutParams.MATCH_PARENT,
                            FrameLayout.LayoutParams.MATCH_PARENT
                        )
                        container.addView(video, lp)
                    }
                }
            }

            1 -> {
                setContentView(R.layout.layout_native_self_render_ad_large_image)
                adContainer = findViewById(R.id.adContainer)
                val largeImage = findViewById<View>(R.id.largeImage) as ImageView
                val closeBtn: ImageView = findViewById<ImageView>(R.id.adCloseBtn)
                val download_item = findViewById<View>(R.id.download_item) as LinearLayout?
                val appName = findViewById<View>(R.id.AppName) as TextView
                val permissions = findViewById<View>(R.id.Permissions) as TextView
                val appVersion = findViewById<View>(R.id.AppVersion) as TextView
                val developerName = findViewById<View>(R.id.DeveloperName) as TextView
                val privacyUrl = findViewById<View>(R.id.PrivacyUrl) as TextView
                if (response.imageUrls != null && response.imageUrls.size !== 0) {
                    val url: String = response.imageUrls[0]
                    GlideTool.load(mContext, url, largeImage)
                }
                // 判断是否是下载类广告
                if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD)
                    if (download_item != null && !TextUtils.isEmpty(response.privacyUrl)) {
                        //demo增加显示下载五要素布局
                        download_item.visibility = View.VISIBLE
                        appName.text = response.downloadAppName
                        appVersion.text = response.appVersion
                        developerName.text = response.developerName
                        //隐私权限点击
                        privacyUrl.setOnClickListener { v: View? ->
                            //点击跳转url加载
//                            NativeAd.UrlSkip(mContext, response.getPrivacyUrl())
                        }
                        //权限说明点击
                        permissions.setOnClickListener { v: View? ->
                            //头条特殊处理，当getPermissionInfoUrl没有返回时调用getPermissionsMap()获取权限键值对进行展示
                            if (!TextUtils.isEmpty(response.permissionInfoUrl)) {
//                                NativeAd.UrlSkip(mContext, response.getPermissionInfoUrl())
                            } else {
                                //权限弹框，需要客户端处理
                                val PermissionsMap: Map<String, String> =
                                    response.permissionsMap
                                //权限弹框
                                NativeAd.CSJPermissionSkip(mContext, PermissionsMap)
                            }
                        }
                    }
                Log.i(TAG, "closeBtn======$closeBtn")
                if (closeBtn != null) {
                    closeBtn.setOnClickListener { v: View? ->
                        Toast.makeText(
                            mContext,
                            "关闭广告",
                            Toast.LENGTH_SHORT
                        ).show()
                        closeNativeSelfRenderAd(response)
                    }
                }
            }

            2 -> {
                setContentView(R.layout.layout_native_self_render_ad_small_image)
                adContainer = findViewById(R.id.adContainer)
                val desc2 = findViewById<View>(R.id.adDesc2) as TextView
                val closeBtn = findViewById<ImageView>(R.id.adCloseBtn)
                val smallImage = findViewById<View>(R.id.smallImage) as ImageView
                val download_item = findViewById<View>(R.id.download_item) as LinearLayout
                val appName = findViewById<View>(R.id.AppName) as TextView
                val permissions = findViewById<View>(R.id.Permissions) as TextView
                val appVersion = findViewById<View>(R.id.AppVersion) as TextView
                val developerName = findViewById<View>(R.id.DeveloperName) as TextView
                val privacyUrl = findViewById<View>(R.id.PrivacyUrl) as TextView
                if (!TextUtils.isEmpty(response.desc)) desc2.text = response.desc
                val url: String = response.imageUrls[0]
                GlideTool.load(mContext, url, smallImage)
                Log.i(TAG, "closeBtn2======$closeBtn")
                closeBtn?.setOnClickListener(View.OnClickListener {
                    Toast.makeText(mContext, "关闭广告", Toast.LENGTH_SHORT).show()
                    closeNativeSelfRenderAd(response)
                })
                //判断是否是下载类广告
                if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD) {
                    if (download_item != null && !TextUtils.isEmpty(response.privacyUrl)) {
                        //demo增加显示下载五要素布局
                        download_item.visibility = View.VISIBLE
                        appName.text = response.downloadAppName
                        appVersion.text = response.appVersion
                        developerName.text = response.developerName
                        //隐私权限点击
                        privacyUrl.setOnClickListener(View.OnClickListener { v: View? ->
                            //点击跳转url加载
//                            NativeAd.UrlSkip(mContext, response.getPrivacyUrl())
                        })
                        //权限说明点击
                        permissions.setOnClickListener(View.OnClickListener { v: View? ->
                            //头条特殊处理，当getPermissionInfoUrl没有返回时调用getPermissionsMap()获取权限键值对进行展示
                            if (!TextUtils.isEmpty(response.permissionInfoUrl)) {
//                                NativeAd.UrlSkip(mContext, response.permissionInfoUrl)
                            } else {
                                val PermissionsMap: Map<String, String> =
                                    response.permissionsMap
                                //权限弹框，需要客户端处理
                                //权限弹框
                                NativeAd.CSJPermissionSkip(mContext, PermissionsMap)
                            }
                        })
                    }
                }
            }

            3 -> {
                setContentView(R.layout.layout_native_self_render_ad_group_image)
                adContainer = findViewById(R.id.adContainer)
                val closeBtn = findViewById<ImageView>(R.id.adCloseBtn)
                val adImage1 = findViewById<View>(R.id.adImage1) as ImageView
                val adImage2 = findViewById<View>(R.id.adImage2) as ImageView
                val adImage3 = findViewById<View>(R.id.adImage3) as ImageView
                val download_item = findViewById<View>(R.id.download_item) as LinearLayout
                val appName = findViewById<View>(R.id.AppName) as TextView
                val permissions = findViewById<View>(R.id.Permissions) as TextView
                val appVersion = findViewById<View>(R.id.AppVersion) as TextView
                val developerName = findViewById<View>(R.id.DeveloperName) as TextView
                val privacyUrl = findViewById<View>(R.id.PrivacyUrl) as TextView
                val imageUrlList: List<String> = response.getImageUrls()
                val imageNum = imageUrlList.size
                if (imageNum > 0) {
                    adImage1.visibility = View.VISIBLE
                    GlideTool.load(mContext, imageUrlList[0], adImage1)
                } else {
                    adImage1.visibility = View.GONE
                }
                if (imageNum > 1) {
                    adImage2.visibility = View.VISIBLE
                    GlideTool.load(mContext, imageUrlList[1], adImage2)
                } else {
                    adImage2.visibility = View.GONE
                }
                if (imageNum > 2) {
                    adImage3.visibility = View.VISIBLE
                    GlideTool.load(mContext, imageUrlList[2], adImage3)
                } else {
                    adImage3.visibility = View.GONE
                }
                Log.i(TAG, "closeBtn3======$closeBtn")
                closeBtn?.setOnClickListener(View.OnClickListener { v: View? ->
                    Toast.makeText(
                        mContext,
                        "关闭广告",
                        Toast.LENGTH_SHORT
                    ).show()
                    closeNativeSelfRenderAd(response)
                })
                //判断是否是下载类广告
                if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD) {
                    if (download_item != null && !TextUtils.isEmpty(response.privacyUrl)) {
                        //demo增加显示下载五要素布局
                        download_item.visibility = View.VISIBLE
                        appName.text = response.downloadAppName
                        appVersion.text = response.appVersion
                        developerName.text = response.developerName
                        //隐私权限点击
                        privacyUrl.setOnClickListener(View.OnClickListener { v: View? ->
                            //点击跳转url加载
//                            NativeAd.UrlSkip(mContext, response.privacyUrl)
                        })
                        //权限说明点击
                        permissions.setOnClickListener(View.OnClickListener { v: View? ->
                            //头条特殊处理，当getPermissionInfoUrl没有返回时调用getPermissionsMap()获取权限键值对进行展示
                            if (!TextUtils.isEmpty(response.permissionInfoUrl)) {
//                                NativeAd.UrlSkip(mContext, response.getPermissionInfoUrl())
                            } else {
                                val PermissionsMap: Map<String, String> =
                                    response.permissionsMap
                                //权限弹框
                                NativeAd.CSJPermissionSkip(mContext, PermissionsMap)
                            }
                        })
                    }
                }
            }

            4 -> {
                setContentView(R.layout.layout_self_render_video_native_ad)
                adContainer = findViewById(R.id.adContainer)
                val container2: RelativeLayout = findViewById<RelativeLayout>(R.id.videoContainer)
                val closeBtn = findViewById<ImageView>(R.id.adCloseBtn)
                val download_item = findViewById<View>(R.id.download_item) as LinearLayout
                val appName = findViewById<View>(R.id.AppName) as TextView
                val permissions = findViewById<View>(R.id.Permissions) as TextView
                val appVersion = findViewById<View>(R.id.AppVersion) as TextView
                val developerName = findViewById<View>(R.id.DeveloperName) as TextView
                val privacyUrl = findViewById<View>(R.id.PrivacyUrl) as TextView
                if (container2 != null) {
                    val videoView: View = response.getVideoView(context)
                    if (videoView != null && videoView.parent !== container2) {
                        val viewGroup = videoView.parent as ViewGroup
                        viewGroup.removeView(videoView)
                        val lp = RelativeLayout.LayoutParams(
                            RelativeLayout.LayoutParams.MATCH_PARENT,
                            RelativeLayout.LayoutParams.MATCH_PARENT
                        )
                        container2.addView(response.getVideoView(context), lp)
                    }
                }
                Log.i(TAG, "closeBtn4==========$closeBtn")
                closeBtn?.setOnClickListener(View.OnClickListener { v: View? ->
                    Toast.makeText(
                        mContext,
                        "关闭广告",
                        Toast.LENGTH_SHORT
                    ).show()
                    closeNativeSelfRenderAd(response)
                })
                if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD) {
                    if (download_item != null && !TextUtils.isEmpty(response.privacyUrl)) {
                        download_item.visibility = View.VISIBLE
                        appName.text = response.downloadAppName
                        appVersion.text = response.appVersion
                        developerName.text = response.developerName
                        privacyUrl.setOnClickListener { v: View? ->
//                            NativeAd.UrlSkip(
//                                mContext,
//                                response.getPrivacyUrl()
//                            )
                        }
                        permissions.setOnClickListener { v: View? ->
                            if (!TextUtils.isEmpty(response.permissionInfoUrl)) {
//                                NativeAd.UrlSkip(mContext, response.permissionInfoUrl)
                            } else {
                                val PermissionsMap: Map<String, String> =
                                    response.permissionsMap
                                //权限弹框
                                NativeAd.CSJPermissionSkip(mContext, PermissionsMap)
                            }
                        }
                    }
                }
            }
        }
        if (type != 0) {
            adPackageName = findViewById<TextView>(R.id.adPackageName)
            title = findViewById<TextView>(R.id.adTitle)
            adIcon = findViewById<ImageView>(R.id.adIcon)
            downloadState = findViewById<TextView>(R.id.adDownload)
            desc = findViewById<View>(R.id.adDesc) as TextView
            adCta = findViewById<TextView>(R.id.adCta)
            updateHeader(response)
            // 注册原生信息流广告可点击区域视图，建议为客户端单个原生广告位所有区域，在广告展现时调用
            (adContainer as ViewGroup?)?.let { response.registerNativeClickableView(it, adContainer) }
        } else {
            (adExpressContainer as ViewGroup?)?.let {
                response.registerNativeClickableView(
                    it,
                    adExpressContainer
                )
            }
        }
        show()
    }

    /**
     * 点击关闭广告按钮，触发此方法刷新信息流列表
     *
     * @param response
     */
    private fun onAdClosed(response: NativeAdsResponse?) {
        Log.i(TAG, "点击了关闭按钮=======$response")
        if (response != null) {
            dismiss()
        }
    }

    /**
     * 关闭原生自渲染广告
     * @param response
     */
    private fun closeNativeSelfRenderAd(response: NativeAdsResponse) {
        Log.i("12345678", "点击了关闭广告")
        dismiss()
        response.onCloseBtnClicked()
    }

    fun updateHeader(response: NativeAdsResponse?) {
        if (response == null) {
            return
        }
        if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_DOWNLOAD) {
            adIcon!!.visibility = View.VISIBLE
            GlideTool.load(mContext, response.logoUrl, adIcon)
        } else if (response.interactionType === AdConfig.NATIVE_INTERACTION_TYPE_LANDINGPAGE) {
            adIcon!!.visibility = View.GONE
            adIcon!!.setImageBitmap(null)
            updateDownloadState("查看详情")
        }
        if (!TextUtils.isEmpty(response.packageName)) {
            adPackageName!!.text = response.packageName
        } else {
            adPackageName!!.text = ""
        }
        if (!TextUtils.isEmpty(response.title) && response.adSource != null) {
            title!!.text = response.adSource.getName() + response.title
        } else if (response.adSource != null) {
            title!!.text = response.adSource.getName()
        }
        if (!TextUtils.isEmpty(response.desc)) {
            desc!!.text = response.desc
        } else {
            desc!!.text = ""
        }
    }

    fun updateDownloadState(state: String?) {
        if (state != null) {
            downloadState!!.visibility = View.VISIBLE
            downloadState!!.text = state
        } else {
            downloadState!!.visibility = View.INVISIBLE
        }
    }

   override fun dismiss() {
        super.dismiss()
        mContext = null
    }
}