package com.example.ydsdkdemo.util;

import android.content.Context;
import android.os.Build;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.view.WindowManager;

public class UIUtils {

    /**
     * px转dp
     *
     * @param context
     * @param pxValue
     * @return
     */
    public static int px2dp(Context context, float pxValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (pxValue / (scale <= 0 ? 1 : scale) + 0.5f);
    }

    /**
     * dp转px
     *
     * @param context
     * @param dp
     * @return
     */
    public static int dp2px(Context context, float dp) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dp * (scale <= 0 ? 1 : scale) + 0.5f);
    }

    /**
     * 返回屏幕尺寸，单位为px
     *
     * @param context
     * @return 数组，分别为宽度和高度
     */
    public static int[] getScreenSizeInPx(Context context) {
        if (context == null) {
            return null;
        }
        DisplayMetrics dm = context.getApplicationContext().getResources().getDisplayMetrics();
        return new int[]{dm.widthPixels, dm.heightPixels};
    }

    /**
     * 返回屏幕尺寸，单位为dp
     *
     * @param context
     * @return 数组，分别为宽度和高度
     */
    public static int[] getScreenSizeInDp(Context context) {
        DisplayMetrics dm = context.getApplicationContext().getResources().getDisplayMetrics();
        return new int[]{px2dp(context, dm.widthPixels), px2dp(context, dm.heightPixels)};
    }

    /**
     * 将视图View从其父容器中移除
     *
     * @param view
     */
    public static void removeFromParent(View view) {
        try {
            if (view != null) {
                ViewParent vp = view.getParent();
                if (vp instanceof ViewGroup) {
                    ((ViewGroup) vp).removeView(view);
                }
            }
        } catch (Throwable t) {
        }
    }

    /**
     * 获取全面屏宽高
     *
     * @param context
     * @return
     */
    public static int[] getScreenRealSizeInPx(Context context) {
        int[] size = new int[]{0, 0};
        if (context == null) {
            return size;
        }
        WindowManager windowManager = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
        Display display = windowManager.getDefaultDisplay();
        DisplayMetrics dm = new DisplayMetrics();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            display.getRealMetrics(dm);
        } else {
            display.getMetrics(dm);
        }
        size[0] = dm.widthPixels;
        size[1] = dm.heightPixels;
        return size;
    }
}
